/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.*;
import java.awt.geom.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.JTabbedPane;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;
import javax.swing.event.*;

/**
 * The AppContainer is a composite class to hold all relevant
 * objects used in the UI.  This allows the ShapePanel to be added to 
 * either an applet or a stand-alone application.
 *
 * Instance variables should be clear after inspecting the interface.
 * @see ShapeApplet
 */
public class AppContainer extends Object {
    Vector<Shape> pgons;

    JSlider smooth, nsides, npieces, psmooth, convex;
    JSlider minsize, maxsize;

    JButton create, tiled;
    JTextField widthField, heightField;
    JCheckBox gridPack;
    JSpinner numColors;
    JSpinner lineNumColors;
    
    JComboBox lineMethodCombo;

    JButton colorButton, lineColorButton;
    JSlider satRange, hueRange, brtRange, spaceSlider, thickness;
    JSlider lineSatRange, lineHueRange, lineBrtRange;

    JCheckBox smoothCheck, nsidesCheck, npiecesCheck, convexCheck, psmoothCheck, lineCheck;
    JCheckBox lineRandom;

    JSlider randomProb, circleProb, squareProb, rectProb, starProb, strokeProb;
    JSlider starMethod, pointMethod, subdivMethod;
    
    JSlider lineRandomProb;
    JSlider tilingNpoints, tilingFrac, boundSmooth;

    /**
     * The ShapePackerThread is a thread to perform the grid-pack operation.
     * This means the user can change the sliders during the grid-pack operation.
     * No way to cancel this thread yet.
     */
    class ShapePackerThread extends Thread {
	public ShapePackerThread(){  }

	public void run(){
	    int width = Integer.valueOf(widthField.getText());
	    int height = Integer.valueOf(heightField.getText());

	    pgons.clear();
	    
	    create.setEnabled(false);
	    
	    ShapePacker tiling = new ShapePacker(width, height);
	    int nfailed = 0;

	    /**
	     * Try to pack as many shapes, until there are 10 successive
	     * failures.  
	     */
	    while (nfailed<10) {
		Shape pgon = createShape();
		pgon.add(new Point2D(1,1));
		double scale = Random.uniform(minsize.getValue(),
					      maxsize.getValue())/2.0;
		if (scale <= 2) continue; 

		//System.out.println("Size: "+sz+" on:"+on);
		boolean found = tiling.packShape(pgon, scale, spaceSlider.getValue());
		
		if (found) {
		    pgon.color = getColor();
		    pgon.lineColor = pgon.color.brighter();
		    nfailed = 0;
		    found = true;
		    pgons.addElement(pgon);
		}
		else {
		    nfailed++;
		}

		// Repaint the panel every few polygons.
		if(pgons.size() % 4 == 0)
		    panel.repaint();
	    }
	    
	    create.setEnabled(true);
	    //setColors();
	}
    }

    /**
     * The ShapePanel is the UI element that draws the shapes.
     * The canvas is scaled to respect the maximum dimensions
     * in the widthField and heightField.
     *
     * BasicShapePanel handles the Drag & Drop events.
     */
    class ShapePanel extends BasicShapePanel {
	public ShapePanel(){ }

	public void paint(Graphics g) {
	    super.paint(g);
	    Graphics2D g2 = (Graphics2D)g;

	    double maxx=0, maxy=0;
	    int w = Integer.valueOf(widthField.getText());
	    int h = Integer.valueOf(heightField.getText());

	    maxx = w;
	    maxy = h;

	    // Preserve aspect ratio, and scale down so that 
	    // all the shapes will fit when drawing.
	    double sc = Math.min(((double)getWidth())/maxx,
				 ((double)getHeight())/maxy);
	    g2.scale(sc, sc);
	    
	    g2.setPaint(new Color(1.f,1.f,1.f,0.3f));
	    g2.fillRect(0,0,w,h);
	    g2.drawRect(0,0,w,h);

	    for(int i=0; i<pgons.size(); i++)
		pgons.elementAt(i).draw(g2);
	}

	public int getCanvasWidth(){
	    return Integer.valueOf(widthField.getText());
	}

	public int getCanvasHeight(){
	    return Integer.valueOf(heightField.getText());
	}

	public Vector<Shape> getShapes(){
	    return pgons;
	}
    }

    /**
     * Helper class to enable a slider when an associated checkbox
     * is changed.
     */
    class Enabler implements ItemListener {
	JSlider slider;
	
	public Enabler(JSlider slider, boolean enabled){
	    this.slider = slider;
	    this.slider.setEnabled(enabled);
	}
	
	public void itemStateChanged(ItemEvent ie){
	    JCheckBox src = (JCheckBox) ie.getSource();
	    slider.setEnabled(src.isSelected());
	    setLineThickness();
	    panel.repaint();
	}
    };


    /**
     * A drop target for handling drops from the MeldShape.
     */
    class MyDropTarget implements DropTargetListener {
	public void dragEnter(DropTargetDragEvent dtde) { }

	public void 	drop(DropTargetDropEvent dtde) {
	    DataFlavor flavs[] = dtde.getCurrentDataFlavors();
	    
	    // Check for the correct type of MIME type.
	    for(int i=0; i<flavs.length; i++){
		System.out.println(flavs[i]);
		System.out.println(flavs[i].getMimeType());

		if (flavs[i].getMimeType()=="text/plain") {
		    System.out.println("got text");
		}	    
	    }

	    dtde.acceptDrop(DnDConstants.ACTION_COPY);

	    try {
		Transferable trans = dtde.getTransferable();
		Object o = trans.getTransferData(new DataFlavor("text/plain; class=java.lang.String; charset=Unicode", "java.lanxg.String"));

		String[] lines = ((String)o).split("\n");
		int startIndex = 0;
		
		// The first line in the file contains the width and height.
		if (lines.length > 0 && lines[0].indexOf("properties ") == 0) {
		    String[] properties = lines[0].substring(11).split(" ");
		    widthField.setText(properties[0]);
		    heightField.setText(properties[1]);
		    startIndex = 1;
		}

		// Add each of the polygons.
		pgons.clear();
		for (int i=startIndex; i<lines.length; i++) {
		    if (lines[i].indexOf("poly") >= 0) {
			Polygon pgon = new Polygon(lines[i].substring(5));
			pgons.addElement(pgon);
		    }
		}
		panel.repaint();
	    }
	    catch (Exception e) {
		System.out.println(e);
	    }	   
	}

	public void dragExit(DropTargetEvent dte) { }

	public void dragOver(DropTargetDragEvent dtde) { }

	public void dropActionChanged(DropTargetDragEvent dtde) { }
    };

    DropTarget dropTarget;

    ShapePanel panel;

    /**
     * Create an AppContainer using the given Container to hold the UI elements.
     * 
     * @param cont   the container that will hold all the UI.
     */
    public AppContainer(Container cont){
	dropTarget = new DropTarget(cont, new MyDropTarget());
	pgons = new Vector<Shape>();
	
	// Add a default Polygon
	pgons.addElement(new Polygon());
	pgons.elementAt(0).add(new Point2D(1,1));
	pgons.elementAt(0).scale(256);


	JPanel menu = new JPanel();
	panel = new ShapePanel();
	JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, menu, panel);

	create = new JButton("Create");
	

	JTabbedPane menuTabs = new JTabbedPane();
	menuTabs.setPreferredSize(new Dimension(10, 300));

	menu.setLayout(new BoxLayout(menu, BoxLayout.Y_AXIS));
	menu.setBorder(BorderFactory.createTitledBorder("Options"));
	menu.add(menuTabs);

	menuTabs.addTab("Basic", createBasicTab());
	menuTabs.addTab("Shapes", createShapesTab());
	menuTabs.addTab("Tiles", createTilingTab());
	menuTabs.add("Color", createColorsTab());
	

	JPanel p = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	p.add(gridPack = new JCheckBox("Grid Pack"));
	p.add(makeRow("space",spaceSlider = new JSlider(JSlider.HORIZONTAL, 0, 40, 0)));
	p.setBorder(BorderFactory.createTitledBorder("Packing:"));
	menu.add(p);

	menu.add(create);

	addSpace(menu);
	menuTabs.setSelectedIndex(0);

	// Add the action listeners.
	create.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    pgons.clear();
		    
		    // Perform a quick inline packing (evenly distributed grid).
		    if (gridPack.isSelected()) { 
			int width  =  Integer.valueOf(heightField.getText())/maxsize.getValue();
			int height =  Integer.valueOf(heightField.getText())/maxsize.getValue();

			for (int i=0; i<width; i++) {
			    for (int j=0; j<height; j++) {
				Shape pgon = createShape();
				
				pgon.add(new Point2D(1,1));
				double scale = Random.uniform(minsize.getValue(),
							      maxsize.getValue())/2.0;
				pgon.scale(scale);
				pgon.add(new Point2D(maxsize.getValue()*i +
						     maxsize.getValue()/2.0-scale,
						     maxsize.getValue()*j +
						     maxsize.getValue()/2.0-scale));

				pgons.addElement(pgon);
			    }
			}
			setColors();
			setLineThickness();
		    }
		    // Perform the Dense packing in a separate thread.
		    else {
			Thread thread = new ShapePackerThread();
			thread.start();
		    }
		}
	    });

	tiled.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    double smoothValue = smoothCheck.isSelected()? 
			((double)smooth.getValue())/100.0 : Random.uniform(0.0, 1.0);
		    double straight = psmoothCheck.isSelected()? 
			(1.0-((double)psmooth.getValue())/100.0) : Random.uniform(0.0, 1.0);
		    double bsmooth = ((double)boundSmooth.getValue())/100.0;
		    double frac = ((double)tilingFrac.getValue())/100.0;
		    double aspect = (double)panel.getCanvasWidth() / (double)panel.getCanvasHeight();
		    
		    try {
			pgons = Tiling.getTiling(tilingNpoints.getValue(),
						 smoothValue, straight, 
						 bsmooth, frac, aspect);

			for (int i=0; i<pgons.size(); i++)
			    ((Shape)pgons.elementAt(i)).scale(Integer.valueOf(heightField.getText()));

			setLineThickness();
			setColors();
		    }
		    catch (Exception e) {
			System.err.println("Error connecting to tile server\n");
			System.err.println(e);

			pgons = new Vector<Shape>();
		    }
		}
	    });

	splitPane.setDividerLocation(200);
	//setContentPane(splitPane);
	//add(splitPane);
	cont.add(splitPane);
    }

    /**
     * Create the tab with the basic options and initialize all the
     * instance variables that are associated with that tab.
     */
    private JPanel createBasicTab() {
	JPanel tab = new JPanel();
	tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));

	convex = new JSlider(JSlider.HORIZONTAL, 0, 100, 25);
	smooth = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
	psmooth = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
	nsides = new JSlider(JSlider.HORIZONTAL, 3, 20, 7);
	npieces = new JSlider(JSlider.HORIZONTAL, 1, 6, 1);
	minsize = new JSlider();
	maxsize = new JSlider();
	thickness = new JSlider(JSlider.HORIZONTAL, 1, 15, 9);
	lineRandomProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 30);

	smooth.setPaintLabels(true);	
	psmooth.setPaintTicks(true);

	minsize.setPaintLabels(true);
	minsize.setMinimum(6);
	minsize.setMaximum(32);
	minsize.setValue(32);

	maxsize.setMinimum(32);
	maxsize.setMaximum(128);
	maxsize.setValue(48);

	tab.add(makeSizeRow());
	tab.add(makeRow("min size",minsize));
	tab.add(makeRow("max size",maxsize));

       	tab.add(makeRow(nsidesCheck = new JCheckBox("nsides"),nsides));
	tab.add(makeRow(npiecesCheck = new JCheckBox("npieces"),npieces));
	tab.add(makeRow(convexCheck = new JCheckBox("convex"),convex));
	tab.add(makeRow(smoothCheck = new JCheckBox("smoothness"),smooth));
	tab.add(makeRow(psmoothCheck = new JCheckBox("% smooth"),psmooth));
	tab.add(makeRow(lineCheck = new JCheckBox("line"), thickness));
	tab.add(makeRow(lineRandom = new JCheckBox("% line"), lineRandomProb));

	npiecesCheck.setSelected(true);
	
	addSpace(tab);


	// Add the action listeners.
	nsidesCheck.addItemListener(new Enabler(nsides, nsidesCheck.isSelected()));
	npiecesCheck.addItemListener(new Enabler(npieces, npiecesCheck.isSelected()));
	convexCheck.addItemListener(new Enabler(convex, convexCheck.isSelected()));
	smoothCheck.addItemListener(new Enabler(smooth, smoothCheck.isSelected()));
	psmoothCheck.addItemListener(new Enabler(psmooth, psmoothCheck.isSelected()));
	lineCheck.addItemListener(new Enabler(thickness, lineCheck.isSelected()));
	lineRandom.addItemListener(new Enabler(lineRandomProb, lineRandom.isSelected()));

	final ChangeListener updateLines = new ChangeListener(){
		public void stateChanged(ChangeEvent ce){
		    setLineThickness();
		    panel.repaint();
		}
	    };
	
	thickness.addChangeListener(updateLines);
	lineRandomProb.addChangeListener(updateLines);

	
	ChangeListener updateBezierChangeListener = new ChangeListener(){
		public void stateChanged(ChangeEvent ce){
		    for (int i=0; i<pgons.size(); i++) {
			pgons.elementAt(i).makeBezier(((double)smooth.getValue())/100.0,
						      ((double)psmooth.getValue())/100.0);
		    }
		    panel.repaint();
		}};

	psmooth.addChangeListener(updateBezierChangeListener);
	smooth.addChangeListener(updateBezierChangeListener);

	return tab;
    }

    /**
     * Create the UI tab representing the shapes panel.
     */
    private JPanel createShapesTab() {
	JPanel tab = new JPanel();
	tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));
	tab.add(makeRow("Random :", randomProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 40)));
	JPanel randMethod = new JPanel();
	randMethod.setLayout(new BoxLayout(randMethod, BoxLayout.Y_AXIS));
	randMethod.setBorder(BorderFactory.createTitledBorder("Random Method"));
	randMethod.add(makeRow("Subdiv", subdivMethod = new JSlider(JSlider.HORIZONTAL, 0, 100, 50)));
	randMethod.add(makeRow("Star  ", starMethod = new JSlider(JSlider.HORIZONTAL, 0, 100, 50)));
	randMethod.add(makeRow("Points", pointMethod = new JSlider(JSlider.HORIZONTAL, 0, 100, 10)));
		
	tab.add(randMethod);
	tab.add(makeRow("Circles:", circleProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 0)));
	tab.add(makeRow("Squares:", squareProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 0)));
	tab.add(makeRow("  Rects:", rectProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 1)));
	tab.add(makeRow("  Stars:", starProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 1)));
	tab.add(makeRow("Strokes:", strokeProb = new JSlider(JSlider.HORIZONTAL, 0, 100, 1)));
	addSpace(tab);
	return tab;
    }

    /**
     * Create the UI tab for the tiling elements.
     */
    private JPanel createTilingTab() {
	JPanel tab = new JPanel();
	tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));
	tab.add(makeRow("# Points:", tilingNpoints = new JSlider(JSlider.HORIZONTAL, 0, 1500, 60)));
	tab.add(makeRow("Frac:", tilingFrac = new JSlider(JSlider.HORIZONTAL, 0, 100, 66)));
	tab.add(makeRow("Bnd-smth:", boundSmooth = new JSlider(JSlider.HORIZONTAL, 0, 100, 66)));
	tab.add(tiled = new JButton("Tile"));
	addSpace(tab);
	return tab;
    }

    /**
     * Create and layout the tab for colors;  also create any associated instance
     * variables.
     */
    private JPanel createColorsTab() {
	JPanel colorPanel = new JPanel();
	colorPanel.setLayout(new BoxLayout(colorPanel, BoxLayout.Y_AXIS));

	JPanel p = new JPanel();
	JPanel row = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	row.add(colorButton = new JButton("    "));
	row.add(new JLabel("#Colors:"));
	row.add(numColors = new JSpinner());
	p.setBorder(BorderFactory.createTitledBorder("color"));

	p.add(row);
	p.add(makeRow("hue range:", hueRange = new JSlider(JSlider.HORIZONTAL,0,100,30) ));
	p.add(makeRow("sat range:", satRange = new JSlider(JSlider.HORIZONTAL,0,100,20) ));
	p.add(makeRow("brt range:", brtRange = new JSlider(JSlider.HORIZONTAL,0,100,20) ));

	addSpace(p);

	SpinnerNumberModel model = new SpinnerNumberModel(1, 1, 5, 1);
	numColors.setModel(model);

	colorButton.setBackground(Color.green.darker());

	// Add change listeners, etc. 
	colorButton.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    final JColorChooser ch = new JColorChooser();
		    JDialog dialog = 
			JColorChooser.createDialog(null, "Choose", true, 
					       ch, 
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						       colorButton.setBackground(ch.getColor());
						       setColors();
						   }
					       }
					       ,
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						   }
					       });
		    dialog.show(true);
		}
	    });

	ChangeListener colorChangeListener = new ChangeListener(){
		public void stateChanged(ChangeEvent ce){
		    setColors();
		    panel.repaint();
		}};
	
	hueRange.addChangeListener(colorChangeListener);
	satRange.addChangeListener(colorChangeListener);
	brtRange.addChangeListener(colorChangeListener);

	colorPanel.add(p);
	
	// Now create the line color.
	p = new JPanel();
	row = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	
	lineMethodCombo = new JComboBox();
	lineMethodCombo.addItem("Darker Color");
	lineMethodCombo.addItem("Lighter Color");
	lineMethodCombo.addItem("Random From Fill");
	lineMethodCombo.addItem("Random");
	lineMethodCombo.addItem("Random Method");
	
	p.add(lineMethodCombo);

       	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));

	row.add(lineColorButton = new JButton("    "));
	row.add(new JLabel("#Colors:"));
	row.add(lineNumColors = new JSpinner());
	p.setBorder(BorderFactory.createTitledBorder("Line Color"));

	p.add(row);
	p.add(makeRow("hue range:", lineHueRange = new JSlider(JSlider.HORIZONTAL,0,100,30) ));
	p.add(makeRow("sat range:", lineSatRange = new JSlider(JSlider.HORIZONTAL,0,100,20) ));
	p.add(makeRow("brt range:", lineBrtRange = new JSlider(JSlider.HORIZONTAL,0,100,20) ));

	addSpace(p);

	model = new SpinnerNumberModel(1, 1, 5, 1);
	lineNumColors.setModel(model);
	
	lineColorButton.setBackground(Color.green.darker());

	// Add change listeners, etc. 
	lineColorButton.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    final JColorChooser ch = new JColorChooser();
		    JDialog dialog = 
			JColorChooser.createDialog(null, "Choose", true, 
					       ch, 
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						       lineColorButton.setBackground(ch.getColor());
						       setLineColors();
						   }
					       }
					       ,
					       new ActionListener(){
						   public void actionPerformed(ActionEvent ae){
						   }
					       });
		    dialog.show(true);
		}
	    });

	colorChangeListener = new ChangeListener(){
		public void stateChanged(ChangeEvent ce){
		    setLineColors();
		    panel.repaint();
		}};
	
	lineHueRange.addChangeListener(colorChangeListener);
	lineSatRange.addChangeListener(colorChangeListener);
	lineBrtRange.addChangeListener(colorChangeListener);

	lineNumColors.addChangeListener(colorChangeListener);

	colorPanel.add(p);


	lineMethodCombo.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    int index = lineMethodCombo.getSelectedIndex();
		    boolean enabled = (index >= 2);

		    lineHueRange.setEnabled(enabled);
		    lineSatRange.setEnabled(enabled);
		    lineBrtRange.setEnabled(enabled);
		    lineNumColors.setEnabled(enabled);
		    lineColorButton.setEnabled(enabled);

		    setLineColors();
		}
	    });

	lineMethodCombo.setSelectedIndex(0);

	return colorPanel;
    }

    /**
     * Return the current color from the UI color settings.
     *
     * @return the current user selected color.
     */
    public Color getColor(){
	ColorSelector selector = new ColorSelector(colorButton.getBackground(),
						   ((float)hueRange.getValue())/100.0f,
						   ((float)satRange.getValue())/100.0f,
						   ((float)brtRange.getValue())/100.0f,
						   Float.valueOf((Integer)numColors.getValue()));
	return selector.getColor();
    }

    /**
     * Set the line thickness for the given shape from the UI widgets
     * 
     * @param s  the shape to set the thickness.
     * @see setLineThickness
     */
    void setLineThickness(Shape s){
    	if(lineRandom.isSelected())
	    s.setThickness((Random.uniform(0, 1) <= (double)lineRandomProb.getValue()/100.0) ?
			   thickness.getValue() : 0);
	else
	    s.setThickness(lineCheck.isSelected() ? 
			   Random.uniform(1, thickness.getValue()) : 0 );
    }

    /**
     * Set the line thickness for every polygon.
     */
    public void setLineThickness(){
	for(int i=0; i<pgons.size(); i++){
	    setLineThickness((Shape)pgons.elementAt(i));
	}
    }


    /**
     * A helper class to choose random colors from a reference color.
     */
    class ColorSelector {
	public ColorSelector(Color _mainColor,
			     float hrg, float satr, float brtr,
			     float ncols) {
	    hueRange = hrg;
	    satRange = satr;
	    brightRange = brtr;
	    numColors = ncols;
	    setMainColor(_mainColor);
	}

	public void setMainColor(Color _mainColor) {
	    mainColor = _mainColor;	    

	    float vals[] = new float[3];
	    Color.RGBtoHSB(mainColor.getRed(), mainColor.getGreen(), mainColor.getBlue(), vals);
	    
	    hue = (float)vals[0];
	    saturation = (float)vals[1];
	    brightness = (float)vals[2];
	}

	public Color getColor() {
	    float h = hue + (float)Random.uniform(-hueRange, hueRange);
	    float s = saturation + (float)Random.uniform(Math.max(-satRange,-saturation), Math.min(satRange, 1.0-saturation));
	    float b = brightness + (float)Random.uniform(Math.max(-brightRange,-brightness), Math.min(brightness, 1.0 - brightness));
	    
	    h = h + (float)Math.floor((float)(Random.uniform(0,1.0)*numColors))/Math.max(numColors, 1.0f);
			
	    s = Math.min(Math.max(s, 0.0f), 1.0f);
	    b = Math.min(Math.max(b, 0.0f), 1.0f);
	    return new Color(Color.HSBtoRGB(h,s,b));
	    // pgons.elementAt(i).lineColor = pgons.elementAt(i).color.brighter();
	}

	public Color mainColor;
	public float hue;
	public float saturation;
	public float brightness;
	public float hueRange;
	public float brightRange;
	public float satRange;
	public float numColors;
	
    };


    /**
     * Set the line colors of each polygon.
     *
     * @todo enumerated type/constant for the method.
     */
    public void setLineColors() {
	ColorSelector selector = new ColorSelector(lineColorButton.getBackground(),
						   ((float)lineHueRange.getValue())/100.0f,
						   ((float)lineSatRange.getValue())/100.0f,
						   ((float)lineBrtRange.getValue())/100.0f,
						   Float.valueOf((Integer)lineNumColors.getValue()));
	int method = lineMethodCombo.getSelectedIndex();

	for(int i=0; i<pgons.size(); i++){
	    Shape shape = pgons.elementAt(i);
	    int methodi = method;

	    if (method == 4) {
		// Generate a number between [0, 3]
		methodi = (int)Random.uniform(0, 3.9999);
	    }

	    if (methodi == 0) {
		shape.lineColor = shape.color.darker();
	    }
	    else if (methodi == 1) {
		shape.lineColor = shape.color.brighter();
	    }
	    else if (methodi == 2) {
		selector.setMainColor(shape.color);
		shape.lineColor = selector.getColor();
	    }
	    else {
		selector.setMainColor(lineColorButton.getBackground());
		shape.lineColor = selector.getColor();
	    }
	}

	panel.repaint();
    }

    /**
     * Set the color for each polygon using the current UI widget values.
     */
    public void setColors(){
	ColorSelector selector = new ColorSelector(colorButton.getBackground(),
						   ((float)hueRange.getValue())/100.0f,
						   ((float)satRange.getValue())/100.0f,
						   ((float)brtRange.getValue())/100.0f,
						   Float.valueOf((Integer)numColors.getValue()));

	for(int i=0; i<pgons.size(); i++){
	    pgons.elementAt(i).color = selector.getColor();
	}

	setLineColors();
	panel.repaint();
    }

    /**
     * Helper function to add extra space to a container.
     *
     * @param p  the container to add the space to.
     */
    private void addSpace(Container p){
	JPanel space = new JPanel();
	space.setPreferredSize(new Dimension(10, 1000));
	p.add(space);
    }

    /**
     * Build a UI row that holds fields for the size parameters.
     * 
     * @return a container holding the UI elemenets for size parameters.
     */
    private Container makeSizeRow(){
	JPanel row = new JPanel();

	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	
	widthField = new JTextField("256", 4);
	heightField = new JTextField("256", 4);
	
	row.add(new JLabel("w:"));
	row.add(widthField);
	row.add(new JLabel("h:"));
	row.add(heightField);

	row.setBorder(BorderFactory.createTitledBorder("Canvas"));
	return row;
    }

    /**
     * Helper function to make and layout a UI row with a label followed
     * by a slider.
     *
     * @param label   the string label to appear on the left of the slider.
     * @param slider  the slider to layout.
     */
    private Container makeRow(String label, JComponent slider){
	return makeRow(new JLabel(label), slider);
    }

    /**
     * Helper function to place a component next to a slider in a
     * single row.
     *
     * @param comp  the component to place on the left.
     * @param slider the slider to place on the right of the row.
     * @return a new UI Container holding the comp and slider.
     */
    private Container makeRow(JComponent comp, JComponent slider){
	JPanel row = new JPanel();
	row.add(comp);
	row.add(slider);
	row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	return row;
    }

    /**
     * Use the current UI settings to build a new shape.
     *
     * @return the newly created shape.
     */
    private Shape createShape(){
	double Pcirc = ((double)circleProb.getValue())/100.0;
	double Prand = ((double)randomProb.getValue())/100.0;
	double Prect = ((double)rectProb.getValue())/100.0;
	double Pstar = ((double)starProb.getValue())/100.0;
	double Pstroke = ((double)strokeProb.getValue())/100.0;
	double Psquare = ((double)squareProb.getValue())/100.0;
	double Ptot = Pcirc+Prand+Prect+Psquare+Pstar+Pstroke;
	
	// Create a random number less than the sum of all probabilities.
	double r = Random.uniform(0.0, Ptot);
	Shape s = null;

	int nsidesUse = nsidesCheck.isSelected()? nsides.getValue(): (int) Math.round(Random.uniform(nsides.getMinimum(), nsides.getMaximum()));

	// Create a circle with probability Pcirc.
	if (r <= Pcirc)
	    s = new Circle(new Point2D(0.0,0.0), 1.0);
	// Create a Polygon with probability Prand
	else if (r <= (Prand+Pcirc)) {
	    Polygon pgon = new Polygon();
	    
	    int npiecesUse = npiecesCheck.isSelected() ? 
		npieces.getValue() : 
		(int) Math.round(Random.uniform(npieces.getMinimum(), npieces.getMaximum()));
	    
	    double smoothUse = smoothCheck.isSelected() ? 
		((double)smooth.getValue())/100.0 : Random.uniform(0.0,1.0);

	    double convexUse = convexCheck.isSelected()
		? ((double)convex.getValue())/100.0 : Random.uniform(0.0,1.0);

	    double psmoothUse = psmoothCheck.isSelected() 
		? ((double)psmooth.getValue())/100.0 : Random.uniform(0.0,1.0);
	
	    // Setup the array that selects probability of a given method
	    double createMethodProbs[] = {((double)starMethod.getValue())/100.0,
					  ((double)pointMethod.getValue())/100.0,
					  ((double)subdivMethod.getValue())/100.0};
	    // Normalize the array of probabilities.
	    double sum = 1e-6;
	    for(int i=0; i<3; i++)
		sum += createMethodProbs[i];

	    for(int i=0; i<3; i++)
		createMethodProbs[i] /= sum;
	    
	    // Create the polygon with the given settings.
	    pgon.create(createMethodProbs, nsidesUse, smoothUse,
			(convexUse == 1.0), npiecesUse,
			psmoothUse, convexUse);
	    s = pgon;
	    
	}
	else if (r <= (Prect+Prand+Pcirc)){
	    Polygon pgon = new Polygon();
	    pgon.createRectangle();
	    s = pgon;
	}
	else if(r <= (Prect+Prand+Pcirc+Psquare)){
	    Polygon pgon = new Polygon();
	    pgon.createSquare();
	    s = pgon;
	}
	else if(r <= (Prect+Prand+Pcirc+Psquare+Pstroke)){
	    Polygon pgon = new Polygon();
	    pgon.createStroke();
	    s = pgon;
	}
	else {
	    Polygon pgon = new Polygon();
	    pgon.createStar(nsidesUse);
	    s = pgon;
	}

	setLineThickness(s);
	return s;
    }

    /**
     * A standalone Swing application that uses the AppContainer
     * to add the UI elements to the contentPane of a frame.
     */
    public static void main(String args[]){
	try {
	    UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
	} 
	catch (Exception evt) {
	    System.out.println("No windows look and feel\n");
	}

	JFrame frame = new JFrame();
	frame.setLayout(new GridLayout(1, 1));
	AppContainer app = new AppContainer(frame.getContentPane());
	
	frame.setSize(new Dimension(640, 460));
	frame.setMinimumSize(new Dimension(640, 460));
	frame.setPreferredSize(new Dimension(640, 460));
	frame.setDefaultCloseOperation(2);
	
	frame.pack();
	frame.show(true);
    }
};
