/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;

import java.awt.*;
import java.awt.geom.*;
import com.nacb.rshape.Contour;

/** 
 * An abstract base class for the objects in the 
 * random shape generator.  Contains colors for 
 * lines and fill, as well as line thickness.
 */
public abstract class Shape{
    static final double MIN_RANDOM_THICKNESS = 1;
    static final double MAX_RANDOM_THICKNESS = 10;

    protected Color color;
    protected Color lineColor;
    protected double thick;
    protected boolean fill;

    /**
     * Construct a basic shape.
     */
    Shape() {
	fill = true;
    }

    /**
     * Draw the shape into the graphics.
     *
     * @param graphics the graphics object to draw on.
     */
    public abstract void draw(Graphics2D graphics);
    
    /**
     * Get a string representation of the shape.
     *
     * @return  the string representation.
     */
    public abstract String getData();

    /**
     * Set the thickness of the shape line.  If thickness
     * is negative, then the thickness is random.
     *
     * @param thickness  the new thickness (negative values imply random)
     */
    public void setThickness(double thickness) {
	if (thickness < 0)
	    this.thick = Random.uniform(MIN_RANDOM_THICKNESS, MAX_RANDOM_THICKNESS);
	else
	    this.thick = thickness;
    }
    
    /**
     * Scale the shape by scale.
     *
     * @param scale the amount to scale
     */
    public abstract void scale(double scale);

    /**
     * Translate the shape by <code>point</code>
     *
     * @param point the amount to translate
     */
    public abstract void add(Point2D pt);

    /**
     * Translate the shape by <code>-point</code>
     *
     * @param point the amount to translate
     */
    public abstract void subtract(Point2D  point);
						
    /**
     * Center the shape.
     */
    public abstract void center();

    /**
     * Make the shape convex.
     * 
     * @see Polygon#makeConvex
     */
    public void makeConvex(){ }

    /**
     * Make the bezier control points for a shape.
     * The default implementation is to do nothing.
     */
    public void makeBezier(double smooth, double minsmth){ }

    /**
     * Return an svg representation of the fill color.
     *
     * @return  svg string for fill color.
     */
    public String getSVGColor() {
	return new String("rgb(" + color.getRed()*100.0/255.0 + "%,"
			  + color.getGreen()*100.0/255.0 + "%,"
			  + color.getBlue()*100.0/255.0 + "%)");
    }

    /**
     * Return an svg representation of the line color.
     *
     * @return  svg string representation of the line color.
     */
    public String getSVGLineColor() {
	return new String("rgb(" +lineColor.getRed()*100.0/255.0 + "%,"
			  + lineColor.getGreen()*100.0/255.0 + "%,"
			  + lineColor.getBlue()*100.0/255.0+"%)");
    }

    /**
     * @return  whether this shape is filled.
     */
    public boolean getFill() {
	return fill;
    }

    /**
     * Set whether this shape is filled.
     * @param  isFilled  whether to fill the shape or not
     */
    public void setFill(boolean isFilled) {
	fill = isFilled;
    }
};