/*
    This file is part of RShape.

    RShape is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RShape is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RShape.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2008-2010 Neil Birkbeck
*/
package com.nacb.rshape;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * Compute a tiling of the plane using a set of connected
 * smooth and non-smooth shapes.  This class currently uses
 * an external program to compute the tiling.  This program originally
 * resided on a web-server, but it can also live locally on the machine.
 *
 * The parameters are passed to the external program, and the resulting
 * polygons are returned as a list of strings.
 *
 * The tiling uses a delaunay triangulation to compute the polygons, followed
 * by some simplication.
 */
class Tiling {
    static final String TILE_URL = "http://neilbirkbeck.homelinux.org/shapes/tiling.php";
    
    /**
     * Compute a tiling of the plane using random shapes.
     * 
     * @param npoints   the number of points to use (these are the corners of points in a delaunay
     *                  triangulation).
     * @param smooth    the amount of smoothness of the final tiling 
     *                  (influences the bezier control points).
     * @param straight  (FIXME) the probability of an edge being smooth.
     * @param bsmooth   the amount of smoothness used on the boundary.
     * @param frac      the amount of reduction that occurs from the original triangulation.
     *                  frac==0 ==> the result will be triangles.
     * @param aspect    the aspect ratio of the plane to be tiled.  The result will
     *                  be a set of polygons within the rectangle [0, aspect] x [0, 1]
     */
    static Vector<Shape> getTiling(int npoints, double smooth, double straight, 
				   double bsmooth, double frac, double aspect) throws Exception {
	Vector<Shape> pgons = new Vector<Shape>();
	BufferedReader reader = null;
	Process proc = null;
	
	try {
	    String str = new String("tiling -n "+ npoints + " -m " + smooth + 
				    " -s " + straight + " -f " + frac + " -b " + 
				    bsmooth + " -a " + aspect);

	    System.out.println("Running " + str);

	    proc = Runtime.getRuntime().exec(str);
	    reader = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	}
	catch(Exception e) {
	    System.out.println("Error starting local process.  Make sure \"tiling\" is on your path."); 
	}

	if(reader == null){
	    URL url = new URL(TILE_URL + "?smooth=" + smooth + "&straight=" + straight + 
			      "&npoints=" + npoints + "&frac=" + frac + "&bsmooth=" + 
			      bsmooth + "&aspect=" + aspect);
	    
	    URLConnection conn = url.openConnection();
	    
	    reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
	}

	String line = null;
	
	pgons.clear();
	while( (line = reader.readLine()) != null){
	    int i = line.indexOf("poly");
	    if(i == 0) {
		Polygon pgon = new Polygon(line.substring(5));
		pgons.addElement(pgon);
	    }
	}
	return pgons;
    }
};